Unit DDEObjects;

Interface

Uses WinTypes, DDEML, Errobj, StrClass;

Type
	PDDEMLObject = ^TDDEMLObject;
	TDDEMLObject = Object( TEObject )
	  f_Inst: LongInt; 					(* ddeml instance *)
		f_Conv: HConv;	 					(* conversation handle *)
		f_Server: HSz;   					(* server name string handle *)
		f_Topic : HSz;   					(* topic name string handle *)
		f_Timeout: LongInt; 			(* timeout for transactions *)
		f_LastResult: LongInt; 		(* last transaction result *)
		f_LastError : Word;    		(* last dde error status *) 
		f_CallbackInst: ^TCallback; (* instance chunk faked as pointer *)
		f_codepage  : Integer; 		(* default code page to use *)

	  Constructor Init( aCallback: TCallback; flags : LongInt );
		Destructor Done; virtual;
		
		Function MakeHSz( aStr: PChar ): HSz;
		Procedure FreeHSz( Var aHSz: HSz );
		Function CompareHSz( HSz1, HSz2: HSz ): Integer;
		Function QueryHSz( aHSz: HSz; Var aStrObj: TStrObj ): Boolean;

		Procedure SetTimeout( newTimeout: LongInt );
		Function  GetTimeout: LongInt;
		Function  GetLastDdeError: longint;
		Function  GetLastDdeResult: longint;
		Function  GetServer: HSz;
		Function  GetTopic: HSz;
		Function  GetCodepage: Integer;
		Procedure SetCodepage( newCodepage: Integer );

		Function Connect( aServer, aTopic: PChar): Boolean;
		Procedure Disconnect;
		Function ClientTransaction( pServerData: Pointer; dataSize: LongInt;
		       					Item: HSz; Format, DataType: Word ): HDDEData;
		Function Request( Item: PChar; Format: Word ): HDDEData;
		Function Execute( command: PChar ): Boolean;
    Function GetData( hData: hDDEData; buf: Pointer; 
    								bufSize, offset: LongInt ): LongInt;
    Function GetDataAsStrObj( hData: hDDEData; Var aStrObj: TStrObj ): Boolean;
		Procedure FreeDatahandle( hData: hDDEData );
		Function CreateDataHandle( source: Pointer; len, offset: LongInt;
															 item: HSz; Format, Command: Word): hDDEData;
		Function AddData( data: hDDEData;source: Pointer;
											len, offset: LongInt ): hDDEData;
    Function AccessData( data: hDDEData; DataSize: PLongint ): Pointer;
		Procedure UnaccessData( data: hDDEData );

		Function IsConnected: Boolean;
		Function IsValid: Boolean;

		Procedure DDE_Error;
		Procedure SetDDE_Error( err: Word );
	End;

Implementation

Uses WinProcs, Strings;

Const
{$I deferrs.inc}	
	ErrOffset = DMLERR_FIRST-1;

function DefDdeCallback(CallType, Fmt: Word; Conv: HConv; hsz1, hsz2: HSZ;
  Data: HDDEData; Data1, Data2: Longint): HDDEData; export;
begin
  DefDdeCallback := 0;
End;


{+------------------------------------------------------------------
 | Methods of TDDEMLObject 
 +------------------------------------------------------------------}

Constructor TDDEMLObject.Init( aCallback: TCallback; flags : LongInt );
  Begin
  	inherited Init;
	  f_Inst:= 0;
		f_Conv:= 0;
		f_Server:= 0;
		f_Topic := 0;
		f_Timeout:= 10000;
		f_LastResult:= 0;
		f_LastError := 0;
		f_CallbackInst:= Nil;
		f_codepage := CP_WINANSI;
		If @aCallback = Nil Then
		  f_CallbackInst := MakeProcInstance( @DefDdeCallback, HInstance )
		Else
		  f_CallbackInst := MakeProcInstance( @aCallback, HInstance );
		If f_CallbackInst = Nil Then
		  Fail;
	  f_LastError := DDEInitialize( f_Inst, TCallback( f_CallbackInst ),
	                   flags, 0 );
		If f_LastError <> DMLERR_NO_ERROR Then
		  Error( errDDEML, f_LastError - ErrOffset );
	End; { TDDEMLObject.Init }

Destructor TDDEMLObject.Done;
  Begin
	  If f_Inst <> 0 Then Begin
		  If f_Conv <> 0 Then 
			  Disconnect;
      DDEUninitialize( f_Inst );
			If f_Server <> 0 Then
			  FreeHSz( f_Server );
			If f_Topic <> 0 Then
			  FreeHSz( f_Topic );
			FreeProcInstance( f_CallbackInst );
		End;
		inherited Done;
	End; { TDDEMLObject.Done }

Function TDDEMLObject.MakeHSz( aStr: PChar ): HSz;
  Var
  	temp: HSz;
  Begin
  	If IsValid Then Begin
		  temp := DdeCreateStringHandle( f_Inst, aStr, f_codepage );
			If temp = 0 Then 
			  DDE_Error;
    End
		Else Begin
			temp := 0;
			SetDDE_Error( dmlErr_DLL_Not_Initialized );
		End;
		MakeHSz := temp;
	End; { TDDEMLObject.MakeHSz }

Procedure TDDEMLObject.FreeHSz( Var aHSz: HSz );
  Begin
  	If aHSz <> 0 Then
		  If IsValid Then Begin
	  		DdeFreeStringHandle( f_Inst, aHSz );
				aHSz := 0;
			End { If }
			Else
			  SetDDE_Error( dmlErr_DLL_Not_Initialized );
	End; { TDDEMLObject.FreeHSz }

Function TDDEMLObject.CompareHSz( HSz1, HSz2: HSz ): Integer;
  Begin
 		CompareHSz := DdeCmpStringHandles( HSz1, HSz2 );
	End; { TDDEMLObject.CompareHSz }

Function TDDEMLObject.QueryHSz( aHSz: HSz; Var aStrObj: TStrObj ): Boolean;
  Var
  	len: LongInt;
  Begin
		QueryHSz := false;
  	If IsValid Then Begin
	  	len := DdeQueryString( f_Inst, aHSz, nil, 0, f_codepage );
			If len = 0 Then 
			  DDE_Error
			Else 
				If not aStrObj.Fits( len, false ) Then
				  ForwardError( aStrObj )
				Else Begin
					DdeQueryString( f_Inst, aHSz, aStrObj.Get, len+1, f_codepage );
					QueryHSz := True;
				End; { Else }
  	End { If }
  	Else 
		  SetDDE_Error( dmlErr_DLL_Not_Initialized );
	End; { TDDEMLObject.QueryHSz }

Procedure TDDEMLObject.SetTimeout( newTimeout: LongInt );
  Begin
	  f_timeout := newTimeout
	End; { TDDEMLObject.SetTimeout }

Function  TDDEMLObject.GetTimeout: LongInt;
  Begin
	  GetTimeout := f_timeout
	End; { TDDEMLObject.GetTimeout }

Function  TDDEMLObject.GetLastDdeError: longint;
  Begin
	  GetLastDdeError := f_lasterror
	End; { TDDEMLObject.GetLastDdeError }

Function  TDDEMLObject.GetLastDdeResult: longint;
  Begin
	  GetLastDdeResult := f_lastresult
	End; { TDDEMLObject.GetLastDdeResult }

Function  TDDEMLObject.GetServer: HSz;
  Begin
	  GetServer := f_server
	End; { TDDEMLObject.GetServer }

Function  TDDEMLObject.GetTopic: HSz;
  Begin
	  GetTopic := f_topic
	End; { TDDEMLObject.GetTopic }

Function  TDDEMLObject.GetCodepage: Integer;
  Begin
	  GetCodepage := f_codepage
	End; { TDDEMLObject.GetCodepage }

Procedure TDDEMLObject.SetCodepage( newCodepage: Integer );
  Begin
	  f_codepage := newCodepage
	End; { TDDEMLObject.SetCodepage }

Function TDDEMLObject.Connect( aServer, aTopic: PChar): Boolean;
  Begin
  	Connect := False;
	  If IsValid Then Begin
		  If IsConnected Then
			  Disconnect;
			If not Failed Then Begin
			  If Assigned( aServer ) Then
			    f_Server := MakeHSz( aServer )
				Else
				  f_Server := 0;
				If Failed Then Exit;
			  If Assigned( aTopic ) Then
			    f_Topic := MakeHSz( aTopic )
				Else
				  f_Topic := 0;
				If Failed Then Exit;
				f_Conv := DdeConnect( f_Inst, f_Server, f_Topic, Nil );
				If f_Conv = 0 Then
				  DDE_Error
				Else
				  Connect := True;
			End; { If }
	  End { If }
		Else
		  SetDDE_Error( dmlErr_DLL_Not_Initialized );
	End; { TDDEMLObject.Connect }

Procedure TDDEMLObject.Disconnect;
  Begin
	  If IsConnected Then Begin
		  DdeDisconnect( f_conv );
			f_Conv := 0;
			FreeHSz( f_Topic );
			FreeHSz( f_Server );
		End;
	End; { TDDEMLObject.Disconnect }

Function TDDEMLObject.ClientTransaction( pServerData: Pointer; dataSize: LongInt;
       					Item: HSz; Format, DataType: Word ): HDDEData;
	Var
		temp: hDDEData;
  Begin
	  temp := 0;
		If IsConnected Then Begin
		  temp := DdeClientTransaction( pServerData, dataSize,
		  						f_Conv, Item, Format, Datatype, f_timeout, 
		  						@f_lastresult );
			If temp = 0 Then
			  DDE_Error;
		End { If }
		Else
		  SetDDE_Error( dmlErr_No_Conv_Established );
		ClientTransaction := temp;
	End; { TDDEMLObject.ClientTransaction }

Function TDDEMLObject.Request( Item: PChar; Format: Word ): HDDEData;
  Var
  	hszItem: HSz;
  Begin
  	hszItem:= MakeHSz( Item );
		If not Failed Then Begin
		  Request := ClientTransaction( Nil, 0, hszItem, Format,
		                XTYP_REQUEST );
			FreeHSz( hszItem );
		End { If }
		Else
		  Request := 0;
	End; { TDDEMLObject.Request }

Function TDDEMLObject.Execute( command: PChar ): Boolean;
  Var
  	data: hDDEData;
  Begin
	  Execute := False;
		If Assigned( command ) Then Begin
			data := CreateDatahandle( command, StrLen( command )+1, 
									0, 0, CF_TEXT, 0 );
			If data <> 0 Then
			  Execute := ClientTransaction( Pointer( data ), cbr_Block,
			  							0, 0, XTYP_EXECUTE ) <> 0;
		End; { If }
	End; { TDDEMLObject.Execute }

Function TDDEMLObject.GetData( hData: hDDEData; buf: Pointer; 
								bufSize, offset: LongInt ): LongInt;
  Begin
	  GetData := DdeGetData( hData, buf, bufSize, offset );
		DDE_Error;
	End; { TDDEMLObject.GetData }

Function TDDEMLObject.GetDataAsStrObj( hData: hDDEData; 
												Var aStrObj: TStrObj ): Boolean;
  Var
  	len: LongInt;
  Begin
  	GetDataAsStrObj := False;
  	If hData <> 0 Then Begin
	  	len := DdeGetData( hData, Nil, 0, 0 );
			If len > 0 Then Begin
			  If aStrObj.Fits( len, false ) Then Begin
				  len := DdeGetData( hData, aStrObj.Get, len, 0 );
					aStrObj.SetLength( len-1 );
					GetDataAsStrObj := True;
			  End { If }
				Else
				  ForwardError( aStrObj );
			End; { If }
		End
		Else
		  SetDDE_Error( dmlErr_InvalidParameter );
	End; { TDDEMLObject.GetDataAsStrObj }

Procedure TDDEMLObject.FreeDatahandle( hData: hDDEData );
  Begin
	  DdeFreeDataHandle( hData );
		DDE_Error;
	End; { TDDEMLObject.FreeDatahandle }

Function TDDEMLObject.CreateDataHandle( source: Pointer; len, offset: LongInt;
													 item: HSz; Format, Command: Word): hDDEData;
  Var 
  	temp: hDDEData;
  Begin
  	temp := 0;
	  If IsValid Then Begin
		  temp := DdeCreateDataHandle( f_Inst, source, len, offset,
		  					item, format, command );
			If temp = 0 Then
			  DDE_Error;
	  End { If }
		Else
		  SetDDE_Error( dmlErr_DLL_Not_Initialized );
		CreateDataHandle := temp;
	End; { TDDEMLObject.CreateDataHandle }

Function TDDEMLObject.AddData( data: hDDEData;source: Pointer;
									len, offset: LongInt ): hDDEData;
  Var
  	temp: hDDEData;
  Begin
	  temp := 0;
		If data <> 0 Then Begin
		  temp := DdeAddData( data, source, len, offset );
			If temp = 0 Then 
			  DDE_Error;
		End; { If }
		AddData := temp;
	End; { TDDEMLObject.AddData }

Function TDDEMLObject.AccessData( data: hDDEData; DataSize: PLongint ): Pointer;
  Var
  	temp: Pointer;
  Begin
	  temp := Nil;
    If data <> 0 Then Begin
		  temp := DdeAccessData( data, DataSize );
			If temp = Nil Then 
			  DDE_Error;
    End; { If }
		AccessData := temp;
	End; { TDDEMLObject.AccessData }

Procedure TDDEMLObject.UnaccessData( data: hDDEData );
  Begin
	  If data <> 0 Then 
		  DdeUnaccessData( data );
	End; { TDDEMLObject.UnaccessData }


Function TDDEMLObject.IsConnected: Boolean;
  Begin
	  IsConnected := f_Conv <> 0;
	End; { TDDEMLObject.IsConnected }

Function TDDEMLObject.IsValid: Boolean;
  Begin
	  IsValid := f_Inst <> 0;
	End; { TDDEMLObject.IsValid }

Procedure TDDEMLObject.DDE_Error;
  Begin
	  SetDDE_Error( DDEGetLastError( f_Inst ));
	End; { TDDEMLObject.DDE }

Procedure TDDEMLObject.SetDDE_Error( err: Word );
  Begin
  	f_lastError := err;
		If err <> dmlErr_No_Error Then
		  Error( errDDEML, err - errOffset )
		Else
		  Reset;
  End; { TDDEMLObject.SetDDE_Error }

End.
