Unit ProgmanDDE;

Interface

Uses WinTypes, Objects, ErrObj, DdeObjects;

Const
	ProgmanServer: PChar = 'PROGMAN';
	ProgmanGroups: PChar = 'GROUPS';

	(* show group flags *)
	SG_RESTORE_ACTIV  = 1;
	SG_MINIMIZE_ACTIV = 2;
	SG_MAXIMIZE_ACTIV = 3;
	SG_RESTORE_NOACTIV= 4;
	SG_ACTIVATE       = 5;
	SG_MINIMIZE       = 6;
	SG_MINIMIZE_NOACTIV = 7;
	SG_SHOW_NOACTIVE  = 8;


Type
	PGroupInfo  = ^TGroupInfo;
	TGroupInfo  = Object( TEObject )
	  f_GroupName : PChar;
		f_GroupFile : PChar;
		f_NumItems  : Integer;
    f_DisplayState: Integer;
		Constructor Init;
		Constructor InitParse( info: PChar );
		Destructor Done; virtual;
		Procedure SetFieldDefaults; virtual;
		Function GetGroupname: PChar;
		Function GetFilename: PChar;
		Function GetNumItems: Integer;
		Function GetDisplayState: Integer;
	End;

	PItemInfo   = ^TItemInfo;
	TItemInfo   = Object( TEObject )
	  f_Title : PChar;
		f_Cmd   : PChar;
		f_Dir   : PChar;
		f_Iconfile: PChar;
		f_PosX, f_PosY: Integer;
		f_IconIndex: Integer;
		f_Hotkey: Word;
		f_Minimize: Boolean;

		Constructor Init;
		Constructor InitParse( info: PChar );
		Constructor InitFull( title, cmd, dir, iconfile: PChar;
		                      posX, posY, iconindex: Integer;
		                      hotkey: Word; minimize: Boolean);
		Procedure SetFieldDefaults; virtual;
		Destructor Done; virtual;
		Procedure Clear; virtual;
	  Function GetTitle: PChar;
		Function GetCmd: PChar;
		Function GetDir: PChar;
		Function GetIconfile: PChar;
		Procedure GetPosition( Var point: TPoint );
		Function GetIconIndex: Integer;
		Function GetHotkey: Word;
		Function GetMinimize: Boolean;
	  Procedure SetTitle( str: PChar );
		Procedure SetCmd( str: PChar );
		Procedure SetDir( str: PChar );
		Procedure SetIconfile( str: PChar );
		Procedure SetPosition( x, y: Integer );
		Procedure SetIconIndex( i: Integer );
		Procedure SetHotkey( key: Word );
		Procedure SetMinimize( f: Boolean );
	End;

	PProgmanDDE = ^TProgmanDDE;
	TProgmanDDE = Object( TDDEMLObject )
	  Constructor Init;
		Destructor Done; virtual;

		Function ConnectToProgman: Boolean;
		Procedure GetGroupnames( Var StrColl: TStrCollection );
		Procedure GetItemsInGroup( group: PChar; Var ItemColl: TCollection;
		 							GroupInfo: PGroupInfo );
		Function IsGroupPresent( group: PChar ): Boolean;
		Function IsItemPresent( group, item: PChar ): Boolean;

		Function CreateGroup( group: PChar ): Boolean;
		Procedure ShowGroup( group: PChar; state: Word );
		Function AddItem( title, command, workdir : PChar ): Boolean;
		Function AddItemFull( Const item: TItemInfo ): Boolean;
		Procedure ReplaceItem( item: PChar );
		Procedure DeleteItem( item: PChar );
		Procedure DeleteGroup( group: PChar );

		(* internal method *)
		Procedure ExecCmd( cmd, param: PChar );
	End;


Implementation

Uses WinProcs, Strings, DDEML, StrClass, Parser;
Const
{$I deferrs.inc}

{+------------------------------------------------------------------
 | Methods of TGroupInfo
 +------------------------------------------------------------------}
Procedure TGroupInfo.SetFieldDefaults;
  Begin
	  f_GroupName :=Nil;
		f_GroupFile :=Nil;
		f_NumItems  :=0;
    f_DisplayState:=0;
	End; { TGroupInfo.SetFieldDefaults }

Constructor TGroupInfo.Init;
  Begin
  	inherited Init;
		SetFieldDefaults;
	End; { TGroupInfo.Init }

Constructor TGroupInfo.InitParse( info: PChar );
  Var
  	parser: PQStrParser;
		pToken: PStrObj;
		p     : PChar;
  Begin
  	inherited Init;
		SetFieldDefaults;
		If not Assigned( info ) Then Exit;
		(* the info string has the form 
		   "Groupname","groupfile",numItems,state *)
		New( parser, Init( info ));
		If Assigned( parser ) Then Begin
		  If not parser^.Failed Then 
			  With parser^ Do Begin
					StartParse;
					p := NextToken;
					If Assigned( p ) Then Begin
						f_GroupName := StrNew( p );
						p := NextToken;
						If Assigned( p ) Then Begin
						  f_GroupFile := StrNew( p );
							pToken := GetTokenObj;
							p := NextToken;
							If Assigned( p ) Then Begin
							  f_NumItems := pToken^.ToInteger;
								p := NextToken;
								If Assigned( p ) Then
								  f_DisplayState := pToken^.ToInteger;
							End; { If }
						End; { If }
					End;
			  End { With }
			Else
			  ForwardError( parser^ );
			Dispose( parser, Done );
		End { If }
		Else
		  Error( errOutOfMemory, Sizeof( TQStrParser ));
	End; { TGroupInfo.InitParse }

Destructor TGroupInfo.Done;
  Begin
	  If Assigned( f_GroupName ) Then
		  StrDispose( f_GroupName );
	  If Assigned( f_GroupFile ) Then
		  StrDispose( f_GroupFile );
		inherited Done;
	End; { TGroupInfo.Done }

Function TGroupInfo.GetGroupname: PChar;
  Begin
	  GetGroupname := f_GroupName
	End; { TGroupInfo.GetGroupname }

Function TGroupInfo.GetFilename: PChar;
  Begin
	  GetFilename := f_GroupFile
	End; { TGroupInfo.GetFilename }

Function TGroupInfo.GetNumItems: Integer;
  Begin
	  GetNumItems := f_NumItems
	End; { TGroupInfo.GetNumItems }

Function TGroupInfo.GetDisplayState: Integer;
  Begin
	  GetDisplayState := f_DisplayState
	End; { TGroupInfo.GetDisplayState }

{+------------------------------------------------------------------
 | Methods of TItemInfo   
 +------------------------------------------------------------------}
Procedure TItemInfo.SetFieldDefaults; 
  Begin
	  f_Title := Nil;
		f_Cmd   := Nil;
		f_Dir   := Nil;
		f_Iconfile:= Nil;
		f_PosX  := 0;
		f_PosY  := 0;
		f_IconIndex:= 0;
		f_Hotkey:= 0;
		f_Minimize:= false;
	End; { TItemInfo.SetFieldDefaults }
	
Constructor TItemInfo.Init;
  Begin
  	inherited Init;
		SetFieldDefaults;
	End; { TItemInfo.Init }

Constructor TItemInfo.InitParse( info: PChar );
  Var
  	parser: PQStrParser;
		pToken: PStrObj;
		p     : PChar;
  Begin
  	inherited Init;
		SetFieldDefaults;
		If not Assigned( info ) Then Exit;
	  New( parser, Init( info ));
		If Assigned( parser ) Then Begin
		  If not parser^.Failed Then 
			  With parser^ Do Begin
					StartParse;
					(* the first item is the item title, it is always present *)
					f_Title := StrNew( NextToken );

					(* the second item is the command string, also always present *)
					f_Cmd := StrNew( NextToken );
          If f_quote <> #0 Then Begin
            Skip;
            f_quote := #0
          End;
					(* now comes the working dir, it may be empty! *)
					Skip;
					If GetCurrChar <> ',' Then
					  f_Dir := StrNew( NextToken );

					(* next is the icon file name. This should be present but we
					   don't count on it! *)
					Skip;
					If GetCurrChar <> ',' Then
					  f_Iconfile := StrNew( NextToken );

					(* following are all numbers, which are always present. We
					   don't check for errors 'cause ToInteger will return 0 in
					   case of errors. *)
					pToken := GetTokenObj;
					NextToken;
					f_posX := pToken^.ToInteger;
					NextToken;
					f_posY := pToken^.ToInteger;
					NextToken;
					f_Iconindex := pToken^.ToInteger;
					NextToken;
					f_Hotkey := pToken^.ToInteger;
					NextToken;
					f_Minimize := pToken^.ToInteger <> 0;
			  End { With }
			Else
			  ForwardError( parser^ );
			Dispose( parser, Done );
		End { If }
		Else
		  Error( errOutOfMemory, Sizeof( TQStrParser ));
	End; { TItemInfo.InitParse }

Constructor TItemInfo.InitFull( title, cmd, dir, iconfile: PChar;
                      posX, posY, iconindex: Integer;
                      hotkey: Word; minimize: Boolean);
  Begin
  	inherited Init;
	  f_Title := StrNew( title );
		f_Cmd   := StrNew( cmd );
		f_Dir   := StrNew( dir );
		f_Iconfile:= StrNew( iconfile );
		f_PosX  := posX;
		f_PosY  := posY;
		f_IconIndex:= iconindex;
		f_Hotkey:= hotkey;
		f_Minimize:= minimize;
	End; { TItemInfo.InitFull }

Destructor TItemInfo.Done; 
  Begin
  	Clear;
		inherited Done;
	End; { TItemInfo.Done }

Procedure TItemInfo.Clear; 
  Begin
	  StrDispose(f_Title);
		StrDispose(f_Cmd);
		StrDispose(f_Dir);
		StrDispose(f_Iconfile);
		f_PosX  := 0;
		f_PosY  := 0;
		f_IconIndex:= 0;
		f_Hotkey:= 0;
		f_Minimize:= false;
	End; { TItemInfo.Clear }

Function TItemInfo.GetTitle: PChar;
  Begin
	  GetTitle := f_Title;
	End; { TItemInfo.GetTitle }

Function TItemInfo.GetCmd: PChar;
  Begin
	  GetCmd := f_Cmd;
	End; { TItemInfo.GetCmd }

Function TItemInfo.GetDir: PChar;
  Begin
	  GetDir := f_Dir;
	End; { TItemInfo.GetDir }

Function TItemInfo.GetIconfile: PChar;
  Begin
	  GetIconfile := f_Iconfile
	End; { TItemInfo.GetIconfile }

Procedure TItemInfo.GetPosition( Var point: TPoint );
  Begin
	  With point Do Begin
		  X := f_posX;
			Y := f_posY;
	  End; { With }
	End; { TItemInfo.GetPosition }

Function TItemInfo.GetIconIndex: Integer;
  Begin
	  GetIconIndex := f_Iconindex
	End; { TItemInfo.GetIconIndex }

Function TItemInfo.GetHotkey: Word;
  Begin
	  GetHotkey := f_hotkey
	End; { TItemInfo.GetHotkey }

Function TItemInfo.GetMinimize: Boolean;
  Begin
	  GetMinimize := f_minimize
	End; { TItemInfo.GetMinimize }

Procedure TItemInfo.SetTitle( str: PChar );
  Begin
	  StrDispose( f_Title );
		f_Title := StrNew( str );
	End; { TItemInfo.SetTitle }

Procedure TItemInfo.SetCmd( str: PChar );
  Begin
	  StrDispose( f_Cmd );
		f_Cmd := StrNew( str );
	End; { TItemInfo.SetCmd }

Procedure TItemInfo.SetDir( str: PChar );
  Begin
	  StrDispose( f_Dir );
		f_Dir := StrNew( str );
	End; { TItemInfo.SetDir }

Procedure TItemInfo.SetIconfile( str: PChar );
  Begin
	  StrDispose( f_Iconfile );
		f_Iconfile := StrNew( str );
	End; { TItemInfo.SetIconfile }

Procedure TItemInfo.SetPosition( x, y: Integer );
  Begin
	  f_posX := x;
		f_posY := y;
	End; { TItemInfo.SetPosition }

Procedure TItemInfo.SetIconIndex( i: Integer );
  Begin
	  f_IconIndex := i
	End; { TItemInfo.SetIconIndex }

Procedure TItemInfo.SetHotkey( key: Word );
  Begin
	  f_Hotkey := key;
	End; { TItemInfo.SetHotkey }

Procedure TItemInfo.SetMinimize( f: Boolean );
  Begin
	  f_Minimize := f;
	End; { TItemInfo.SetMinimize }

{+------------------------------------------------------------------
 | Methods of TProgmanDDE 
 +------------------------------------------------------------------}
Constructor TProgmanDDE.Init;
  Begin
	  inherited Init( Nil, APPCMD_CLIENTONLY or CBF_SKIP_ALLNOTIFICATIONS );
	End; { TProgmanDDE.Init }

Destructor TProgmanDDE.Done; 
  Begin
	  inherited Done;
	End; { TProgmanDDE.Done }

Function TProgmanDDE.ConnectToProgman: Boolean;
  Begin
	  ConnectToProgman := Connect( ProgmanServer, ProgmanServer );
	End; { TProgmanDDE.ConnectToProgman }

Procedure TProgmanDDE.GetGroupnames( Var StrColl: TStrCollection );
  Var
  	data: hDDEData;
		len : Word;
		parser: PStrParser;
		p   : PChar;
		n   : Integer;
  Begin
  	StrColl.FreeAll;
	  If IsConnected or ConnectToProgman Then Begin
		  data := Request( ProgmanGroups, CF_TEXT );
      If data <> 0 Then Begin
			  len := LoWord( GetData( data, nil, 0, 0 ));
				New( parser, InitSize( len ));
				If Assigned( parser ) Then 
					With parser^ Do Begin
					  If not parser^.Failed Then Begin
						  GetData( data, Get, len, 0 );
              SetLength( StrLen(Get));
						  SetSeparators( [ #0..#31 ] );
							StartParse;
							p := NextToken;
							While Assigned( p ) Do Begin
							  If not StrColl.Search( p, n ) Then
								  StrColl.AtInsert( n, StrNew( p ));
								p := NextToken;
							End; { While }
						End
						Else
						  ForwardError( parser^ );
					  Dispose( parser, Done );
					End
				Else
				  Error( errOutOfMemory, Sizeof( TStrParser ));
				FreeDataHandle( data );
      End; { If }
	  End; { If }
	End; { TProgmanDDE.GetGroupnames }

Procedure TProgmanDDE.GetItemsInGroup( group: PChar; 
              Var ItemColl: TCollection;
 							GroupInfo: PGroupInfo );
  Var
  	data: hDDEData;
		len : Word;
		parser: PStrParser;
		p   : PChar;
		n   : Integer;
		pItem: pItemInfo;
  Begin
  	ItemColl.FreeAll;
	  If IsConnected or ConnectToProgman Then Begin
		  data := Request( group, CF_TEXT );
      If data <> 0 Then Begin
			  len := LoWord( GetData( data, nil, 0, 0 ));
				New( parser, InitSize( len ));
				If Assigned( parser ) Then
					With parser^ Do Begin
					  If not parser^.Failed Then Begin
						  GetData( data, Get, len, 0 );
              SetLength( StrLen(Get));
						  SetSeparators( [ #0..#31 ] );
							StartParse;
							(* the first line contains the group info *)
							p := NextToken;
							If Assigned( GroupInfo ) Then
							  GroupInfo^.InitParse( p );
							(* the following lines are item infos *)
							p:= NextToken;
							While Assigned( p ) Do Begin
							  New( pItem, InitParse( p ));
								If Assigned( pItem ) Then
								  If pItem^.Failed Then Begin
									  Dispose( pItem, Done );
									  Break
									End
									Else
									  ItemColl.Insert( pItem )
								Else Begin
								  Error( errOutOfMemory, Sizeof( TItemInfo ));
								  Break;
								End;
								p := NextToken;
							End; { While }
						End
						Else
						  ForwardError( parser^ );
					  Dispose( parser, Done );
					End { With }
				Else
					Error( errOutOfMemory, Sizeof( TStrParser ));
				FreeDataHandle( data );
      End; { If }
	  End; { If }
	End; { TProgmanDDE.GetItemsInGroup }

Function TProgmanDDE.IsGroupPresent( group: PChar ): Boolean;
  Var
  	Coll: PStrCollection;
	Function Matches( p: PChar ): Boolean; far;
		Begin
			Matches := StrIComp( p, group ) = 0;
		End; { Matches }
  Begin
  	IsGroupPresent := False;
	  If Assigned( group ) Then Begin
		  New( Coll, Init( 25, 10 ));
			If Assigned( Coll ) Then Begin
			  GetGroupNames( Coll^ );
				IsGroupPresent := Coll^.FirstThat( @Matches ) <> Nil;
			  Dispose( Coll, Done );
			End { If }
			Else
			  Error( errOutOfMemory, 
			  			 Sizeof( TStrCollection )+25*Sizeof( Pointer ));
	  End; { If }
	End; { TProgmanDDE.IsGroupPresent }

Function TProgmanDDE.IsItemPresent( group, item: PChar ): Boolean;
  Var
  	Coll: PCollection;
	Function Matches( p: PItemInfo ): Boolean; far;
		Begin
			Matches := StrIComp( p^.GetTitle, item ) = 0;
		End; { Matches }
  Begin
	  IsItemPresent := False;
	  If Assigned( group ) and Assigned( item ) Then Begin
		  New( Coll, Init( 50, 10 ));
			If Assigned( Coll ) Then Begin
			  GetItemsInGroup( group, Coll^, Nil );
				IsItemPresent := Coll^.FirstThat( @Matches ) <> Nil;
			  Dispose( Coll, Done );
			End { If }
			Else
			  Error( errOutOfMemory, 
			  			 Sizeof( TStrCollection )+50*Sizeof( Pointer ));
	  End; { If }
	End; { TProgmanDDE.IsItemPresent }

Function TProgmanDDE.CreateGroup( group: PChar ): Boolean;
  Begin
  	CreateGroup := False;
		If Assigned( group ) Then Begin
		  ExecCmd( '[CreateGroup("%s")]', group );
			CreateGroup := IsGroupPresent( group );
	  End;
	End; { TProgmanDDE.CreateGroup }

Procedure TProgmanDDE.ShowGroup( group: PChar; state: Word );
  Var
  	sObj: TStrObj;
		temp: Record p: PChar; w: Word End;
  Begin
		If Assigned( group ) Then
		  If IsConnected or ConnectToProgman Then Begin
			  sObj.InitBlank;
				temp.p := group;
				temp.w := state;
				sObj.PutFormat( '[ShowGroup("%s",%i)]', temp );
				If sObj.Failed Then
				  ForwardError( sObj )
				Else 
					Execute( sObj.Get );
				sObj.Done;
		  End;
	End; { TProgmanDDE.ShowGroup }

Function TProgmanDDE.AddItem( title, command, workdir : PChar ): Boolean;
  Const
  	s: PChar = '%s';
  Var
  	sObj: TStrObj;
  Begin
  	AddItem := False;
		If Assigned( title ) and Assigned( command ) Then
		  If IsConnected or ConnectToProgman Then Begin
			  sObj.InitSize( 128 );
				If sObj.Failed Then
				  ForwardError( sObj )
				Else Begin
				  If Assigned( workdir ) Then
				  	sObj.Put( '[AddItem("%s","%s",,,,,%s)]' )
					Else
					  sObj.Put( '[AddItem("%s","%s")]' );
					sObj.Replace( s, command );
					sObj.Replace( s, title );
					If Assigned( workdir ) Then
				  	sObj.Replace( s, workdir );
					AddItem := Execute( sObj.Get );
					sObj.Done;
				End; { Else }
		  End;
	End; { TProgmanDDE.AddItem }

Function TProgmanDDE.AddItemFull( Const item: TItemInfo ): Boolean;
  Var
  	sObj: TStrObj;
  Begin
  	AddItemFull := False;
		If Assigned( item.f_title ) and Assigned( item.f_cmd ) Then
		  If IsConnected or ConnectToProgman Then Begin
			  sObj.InitSize( 255 );
				If sObj.Failed Then
				  ForwardError( sObj )
				Else Begin
				  sObj.Put( '[AddItem("' );
					sObj.Append( item.f_cmd );
					sObj.Append( '","' );
					sObj.Append( item.f_title );
					sObj.Append( '",' );
					If Assigned( item.f_Iconfile ) Then Begin
					  sObj.Append( item.f_Iconfile );
						sObj.AppendChar( ',' );
						sObj.AppendInteger( item.f_IconIndex, 0 );
						sObj.AppendChar( ',' );
					End
					Else
					  sObj.Append( ',,' );
					If item.f_posX > 0 Then Begin
					  sObj.AppendInteger( item.f_PosX, 0 );
						sObj.AppendChar( ',' );
					  sObj.AppendInteger( item.f_PosY, 0 );
						sObj.AppendChar( ',' );
					End { If }
					Else 
					  sObj.Append( ',,' );
					If Assigned( item.f_Dir ) Then
					  sObj.Append( item.f_Dir );
					sObj.AppendChar( ',' );
					If item.f_hotkey <> 0 Then 
					  sObj.AppendInteger( item.f_hotkey, 0 );
					sObj.AppendChar( ',' );
					If item.f_minimize Then
					  sObj.AppendChar( '1' )
					Else
					  sObj.AppendChar( '0' );
					sObj.Append( ')]' );
					AddItemFull := Execute( sObj.Get );
					sObj.Done;
				End; { Else }
		  End;
	End; { TProgmanDDE.AddItemFull }

Procedure TProgmanDDE.ReplaceItem( item: PChar );
  Begin
		If Assigned( item ) Then
			ExecCmd( '[ReplaceItem("%s")]', item );
	End; { TProgmanDDE.ReplaceItem }

Procedure TProgmanDDE.DeleteItem( item: PChar );
  Begin
		If Assigned( item ) Then 
			ExecCmd( '[DeleteItem("%s")]', item );
	End; { TProgmanDDE.DeleteItem }

Procedure TProgmanDDE.DeleteGroup( group: PChar );
  Begin
		If Assigned( group ) Then 
			ExecCmd( '[DeleteGroup("%s")]', group );
	End; { TProgmanDDE.DeleteGroup }

Procedure TProgmanDDE.ExecCmd( cmd, param: PChar );
  Var
  	sObj: TStrObj;
  Begin
	  If IsConnected or ConnectToProgman Then Begin
		  sObj.InitBlank;
			sObj.PutFormat( cmd, param );
			If sObj.Failed Then
			  ForwardError( sObj )
			Else 
				Execute( sObj.Get );
			sObj.Done;
	  End;
	End; { TProgmanDDE.ExecCmd }

End.
