{+------------------------------------------------------------
 | Program ProgManSnoop
 |
 | Version: 1.0  Last modified: 03/03/95, 17:02:33
 | Author : P. Below
 | Project: Sample program for ProgmanDDE Object
 | Description:
 |   This is a sample program to show the workings of the ProgmanDDE
 |	 object. I displays a simple dialog with all groups defined in
 |	 Progman in a list and all items in the currently selected group
 |	 in another list. Detail info for the current group and item
 |	 is displayed in a couple of statics. User can select groups 
 |	 and items to show info for.
 +------------------------------------------------------------}
Program ProgManSnoop;

Uses WinTypes, WinProcs, Win31, Objects, OWindows, ODialogs, Strings,
     ErrObj, ProgmanDDE, GrayDlgs, StrClass, ErrColl;

{$R psnoop.res}
{$D Copyright (c) 1995 by Peter Below}

{$I psnoop.inc}
{$I deferrs.inc}
Const
  AppName : PChar = 'ProgManSnoop';
	UM_QUERYGROUPS = WM_USER+101;
	UM_QUERYITEMS  = WM_USER+102;
Type
  TMyApplication = Object(TApplication)
    Procedure InitMainWindow; virtual;
  End;

  PMainDialog = ^TMainDialog;
  TMainDialog = Object(TGrayDlgWindow)
	  waiting: Boolean;
		aPMTask: TProgmanDDE;
		theGroupList: TEStrCollection;
		theCurrentSelection: Integer;
		theItemList : TECollection;
		theGroupListbox: PListbox;
		theItemListbox : PListbox;
		theCurrentGroup: TGroupInfo;
		theCurrentItem : PItemInfo;

    Constructor Init;
    Destructor Done; Virtual;
    Procedure SetUpWindow; Virtual;
    Function GetClassName : PChar; Virtual;
    Procedure GetWindowClass(Var AWndClass : TWndClass);
      Virtual;

		Procedure ClearGroupDetails;
		Procedure ClearItemDetails;
		Procedure ShowGroupDetails;
		Procedure ShowItemDetails;
		Function FindItem( itemName: PChar ): pItemInfo; 

		Procedure OnGroupSelect( Var msg: TMessage );
			virtual id_first+ID_GROUPLIST;
		Procedure OnItemSelect( Var msg: TMessage );
			virtual id_first+ID_ICONLIST;
		Procedure WMSetCursor( Var msg: TMessage );
			virtual WM_FIRST+WM_SETCURSOR;
		Procedure OnQueryGroups( Var msg: TMessage );
			virtual WM_FIRST+UM_QUERYGROUPS;
		Procedure OnQueryItems( Var msg: TMessage );
			virtual WM_FIRST+UM_QUERYITEMS;

  End;

{+--------------------------------------------------
 | Methods of TMainDialog 
 +--------------------------------------------------}
Constructor TMainDialog.Init;
  Begin
    inherited Init(Nil, MakeIntResource( IDD_MAIN ));
		aPMTask.Init;
		theGroupList.Init( 20,10 );
		theItemList.Init( 50,10 );
		New( theGroupListbox, InitResource( @Self, ID_GROUPLIST ));
		New( theItemListbox, InitResource( @Self, ID_ICONLIST ));
		theCurrentGroup.Init;
		theCurrentItem  := Nil;
		waiting := False;
		theCurrentSelection := -1;
  End; { TMainDialog.Init }

Procedure TMainDialog.SetUpWindow;
  Begin
    inherited SetUpWindow;
		ClearGroupDetails;
		ClearItemDetails;
		PostMessage( HWIndow, UM_QUERYGROUPS, 0, 0 );
  End; { TMainDialog.SetUpWindow }

Destructor TMainDialog.Done;
  Begin
  	aPMTask.Done;
		theGroupList.Done;
		theItemList.Done;
    theCurrentGroup.Done;
    inherited Done;
  End; { TMainDialog.Done }

Function TMainDialog.GetClassName;
  Begin
    GetClassName := AppName;
  End; { TMainDialog.GetClassName }

Procedure TMainDialog.GetWindowClass(Var AWndClass: TWndClass);
  Begin
    inherited GetWindowClass(AWndClass);
    AWndClass.hIcon := 0;
		AWndClass.hCursor := 0;
  End; { TMainDialog.GetWindowClass }

Procedure TMainDialog.ClearGroupDetails;
  Const
  	ids: Array [ 1..4 ] of Word = 
  	  ( ID_GROUPNAME, ID_GROUPFILE, ID_GROUPITEMS, ID_GROUPSTATE );
  Var
  	c: Char;
		i: Integer;
  Begin
	  c := #0;
		For i:= 1 To 4 Do 
		  SetDlgItemText( HWindow, ids[ i ], @c );
	End; { TMainDialog.ClearGroupDetails }

Procedure TMainDialog.ClearItemDetails;
  Const
  	ids: Array [ 1..9 ] of Word = ( 
             ID_ICONTITLE, ID_ICONCOMMAND, ID_ICONWORKDIR, ID_ICONFILE,
						 ID_ICONPOSX, ID_ICONPOSY, ID_ICONINDEX, ID_ICONHOTKEY, 
						 ID_ICONMINFLAG	 );
  Var
  	c: Char;
		i: Integer;
  Begin
	  c := #0;
		For i:= 1 To 9 Do 
		  SetDlgItemText( HWindow, ids[ i ], @c );
	End; { TMainDialog.ClearItemDetails }

Procedure TMainDialog.ShowGroupDetails;
  Begin
	  With theCurrentGroup Do Begin
		  SetDlgItemText( HWindow, ID_GROUPNAME, GetGroupname );
		  SetDlgItemText( HWindow, ID_GROUPFILE, GetFilename );
			SetDlgItemInt( HWindow, ID_GROUPITEMS, GetNumItems, TRUE );
			SetDlgItemInt( HWindow, ID_GROUPSTATE, GetDisplayState, TRUE );
	  End; { With }
	End; { TMainDialog.ShowGroupDetails }

Procedure TMainDialog.ShowItemDetails;
  Var
  	pnt: TPoint;
  Begin
  	If Assigned( theCurrentItem ) Then
		  With theCurrentItem^ Do Begin
			  SetDlgItemText( HWindow, ID_ICONTITLE, GetTitle );
			  SetDlgItemText( HWindow, ID_ICONCOMMAND, GetCmd );
			  SetDlgItemText( HWindow, ID_ICONWORKDIR, GetDir );
			  SetDlgItemText( HWindow, ID_ICONFILE, GetIconfile );
				GetPosition( pnt );
				SetDlgItemInt( HWindow, ID_ICONPOSX, pnt.X, TRUE );
				SetDlgItemInt( HWindow, ID_ICONPOSY, pnt.Y, TRUE );
				SetDlgItemInt( HWindow, ID_ICONINDEX, GetIconIndex, TRUE );
				SetDlgItemInt( HWindow, ID_ICONHOTKEY, GetHotKey, FALSE );
				If GetMinimize Then
			  	SetDlgItemText( HWindow, ID_ICONMINFLAG, 'Yes' )
				Else
			  	SetDlgItemText( HWindow, ID_ICONMINFLAG, 'No' );
		  End { With }
		Else
		  ClearItemDetails;
	End; { TMainDialog.ShowItemDetails }

Function TMainDialog.FindItem( itemName: PChar ): pItemInfo; 
  Function Matches( p: PItemInfo ): Boolean; far;
	  Begin
	  	Matches := False;
			If Assigned( p ) Then 
			  matches := StrComp( itemName, p^.GetTitle ) = 0;
	  End;
  Begin
	  FindItem := Nil;
		If Assigned( itemName ) and ( StrLen( itemName  ) > 0) Then
		  FindItem:= theItemList.FirstThat( @Matches );
	End; { TMainDialog.FindItem }

Procedure TMainDialog.OnGroupSelect( Var msg: TMessage );
  Var
  	i: Integer;
  Begin
	  If msg.lparamhi = LBN_SELCHANGE Then Begin
		  i:= theGroupListbox^.GetSelIndex;
		  If ( i <> theCurrentSelection ) and ( i >= 0 ) Then Begin
				theCurrentSelection := i;
				PostMessage( HWindow, UM_QUERYITEMS, 0, 0 );
		  End; { If }
	  End; { If }
		msg.result := 0;
	End; { TMainDialog.OnGroupSelect }

Procedure TMainDialog.OnItemSelect( Var msg: TMessage );
  Var
  	i: Integer;
		aStr: TStrObj;
  Begin
	  If msg.lparamhi = LBN_SELCHANGE Then Begin
		  i:= theItemListbox^.GetSelIndex;
		  If i >= 0 Then Begin
			  aStr.InitSize( theItemListbox^.GetStringLen( i ));
        theItemListbox^.GetSelString( aStr.Get, aStr.GetCapacity );
			  theCurrentItem := FindItem( aStr.Get );
				aStr.Done;
				ShowItemDetails;
		  End; { If }
	  End; { If }
		msg.result := 0;
	End; { TMainDialog.OnItemSelect }

Procedure TMainDialog.WMSetCursor( Var msg: TMessage );
  Begin
	  If waiting Then Begin
		  SetCursor( LoadCursor( 0, IDC_WAIT ));
			msg.result := 1;
			SetWindowLong(hWindow, DWL_MSGRESULT, 1);
	  End { If }
		Else
			If msg.lparamlo = HTCLIENT Then Begin
			  SetCursor( LoadCursor( 0, IDC_ARROW ));
				msg.result := 0;
				SetWindowLong(hWindow, DWL_MSGRESULT, 0);
			End  { If }
			Else
			  DefWndProc( msg );
	End; { TMainDialog.WMSetCursor }

Procedure TMainDialog.OnQueryGroups( Var msg: TMessage );
  Var
  	transferRec : Record
		   pStr: PStrCollection;
			 sel : Integer;
		End;
  Begin
	  waiting := True;
		SetCursor( LoadCursor( 0, IDC_WAIT ));
		theGroupList.FreeAll;
		aPMTask.GetGroupNames( theGroupList );
		theCurrentSelection := 0;
		With transferRec Do Begin
		  pStr := @theGroupList;
			sel  := theCurrentSelection;
		End; { With }
		theGroupListbox^.Transfer( @transferRec, tf_SetData );
		waiting := False;
		SetCursor( LoadCursor( 0, IDC_ARROW ));
		PostMessage( Hwindow, UM_QUERYITEMS, 0, 0 );
	End; { TMainDialog.OnQueryGroups }

Procedure TMainDialog.OnQueryItems( Var msg: TMessage );
  Var
  	transferRec : Record
		   pStr: PStrCollection;
			 sel : Integer;
		End;
	Procedure AddItem( p: PItemInfo ); far;
		Begin
			If Assigned( p ) Then
			  transferRec.pStr^.Insert( StrNew( p^.GetTitle ));
		End; { AddItem }
  Begin
	  waiting := True;
		SetCursor( LoadCursor( 0, IDC_WAIT ));
		theItemList.FreeAll;
		aPMTask.GetItemsInGroup( theGroupList.At( theCurrentSelection ),
		   											 theItemList, @theCurrentGroup );
		New( transferRec.pStr, Init( 50, 10 ));
		theItemList.ForEach( @AddItem );
		transferRec.sel := 0;
		theItemListbox^.Transfer( @transferRec, tf_SetData );
		theCurrentItem := FindItem( transferRec.pStr^.At( 0 ));
		ShowGroupDetails;
		ShowItemDetails;
		Dispose( transferRec.pStr, Done );
		waiting := False;
		SetCursor( LoadCursor( 0, IDC_ARROW ));
	End; { TMainDialog.OnQueryItems }


{--------------------------------------------------}
{ TMyApplication's method implementations:         }
{--------------------------------------------------}
  Procedure TMyApplication.InitMainWindow;
  Begin
    MainWindow := New(PMainDialog, Init);
  End;


Procedure GlbError( code, info: Integer ); far;
  Const
    eMask: PChar = 'Error detected!'#13#10+
                   '   Error Code: %i'#13#10+
                   '   Error Info: %i';
  Var
    eStr: Array [0..128] of Char;
    temp: Record i1,i2: Integer End;
  Begin
    temp.i1 := code;
    temp.i2 := info;
    wvsprintf( eStr, eMask, temp );
    MessageBox( GetActiveWindow, eStr, 'PSnoop Global Error Handler',
                MB_OK or MB_ICONSTOP );

  End;

{--------------------------------------------------}
{ Main Program:                                    }
{--------------------------------------------------}
Var MyApp: TMyApplication;
Begin
  SetGlobalErrorProc( GlbError );
  MyApp.Init(AppName);
  MyApp.Run;
  MyApp.Done;
End.

